<?php
/**
 * @version 1.0 $Id: com_quickfaq.php 142 2008-08-05 03:27:17Z schlu $
 * @package Joomla
 * @subpackage QuickFAQ Xmap plugin
 * @copyright (C) 2008 Christoph Lukes
 * @license GNU/GPL, see LICENCE.php
 * QuickFAQ is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.

 * QuickFAQ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with QuickFAQ; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

require_once (JPATH_SITE.DS.'components'.DS.'com_quickfaq'.DS.'helpers'.DS.'route.php');
require_once (JPATH_SITE.DS.'components'.DS.'com_quickfaq'.DS.'classes'.DS.'quickfaq.acl.php');

class xmap_com_quickfaq {

	/*
	 * Base entry
	 */
	function getTree( &$xmap, &$parent, &$params) {
	
		//prevent parsing of favourites list and single faqitem menu items
		if ( strpos($parent->link, 'view=favourites') || strpos($parent->link, 'view=items') ) {
			return;	
		}

		$link_query = parse_url( $parent->link );
		parse_str( html_entity_decode( $link_query['query']), $link_vars );
		$catid = JArrayHelper::getValue( $link_vars, 'cid', 0 );
		$tid = JArrayHelper::getValue( $link_vars, 'id', 0 );

		$include_items = JArrayHelper::getValue( $params, 'include_items',1,'' );
		$include_items = ( $include_items == 1
						|| ( $include_items == 2 && $xmap->view == 'xml') 
						|| ( $include_items == 3 && $xmap->view == 'html'));
		$params['include_items'] = $include_items;

		//Category settings
		$priority 	= JArrayHelper::getValue( $params, 'cat_priority', $parent->priority, '' );
		$changefreq = JArrayHelper::getValue( $params, 'cat_changefreq', $parent->changefreq, '' );
		
		if ($priority  == '-1') {
			$priority = $parent->priority;
		}
		
		if ($changefreq  == '-1') {
			$changefreq = $parent->changefreq;
		}

		$params['cat_priority'] 	= $priority;
		$params['cat_changefreq'] 	= $changefreq;

		//Item settings
		$priority 	= JArrayHelper::getValue( $params, 'item_priority', $parent->priority, '' );
		$changefreq = JArrayHelper::getValue( $params, 'item_changefreq', $parent->changefreq, '' );
		
		if ($priority  == '-1') {
			$priority = $parent->priority;
		}

		if ($changefreq  == '-1') {
			$changefreq = $parent->changefreq;
		}

		$params['item_priority'] 	= $priority;
		$params['item_changefreq'] 	= $changefreq;

		if ( $include_items ) {
			$params['limit'] 	= '';
			$params['days'] 	= '';
			$limit 				= JArrayHelper::getValue( $params, 'max_items', '', '' );

			if ( intval($limit) ) {
				$params['limit'] = ' LIMIT '.(int)$limit;
			}
			
			$days = JArrayHelper::getValue( $params, 'max_age', '', '' );
			
			if ( intval($days) ) {
				$params['days'] = ' AND i.created >= \''.date('Y-m-d H:m:s', ($xmap->now - ($days*86400)) ) ."' ";
			}
		}
		
		//Get the data
		if (strpos($parent->link, 'view=tags')) {
			xmap_com_quickfaq::getQuickfaqTagTree( $xmap, $parent, $params, $tid );
		} else {
			xmap_com_quickfaq::getQuickfaqCatTree( $xmap, $parent, $params, $catid );
		}
	}

	/*
	 * Get the Categories hierarchie and assigned FAQItems
	 */
	function getQuickfaqCatTree ( &$xmap, &$parent, &$params, &$catid ) {
		
		$db 		= &JFactory::getDBO();
		$user 		= &JFactory::getUser();
		$gid		= (int) $user->get('aid');
		
		$xmap->changeLevel(1);
		
		//dont fetch parentcategories when parsing an categoryview menu item
		if (!strpos($parent->link, 'view=category')) {
		
			$query = 'SELECT c.id, c.title, c.parent_id,'
					. ' CASE WHEN CHAR_LENGTH( c.alias ) THEN CONCAT_WS( \':\', c.id, c.alias ) ELSE c.id END AS slug'
					. ' FROM #__quickfaq_categories AS c'
					. ' WHERE c.published = 1'
					. ' AND c.parent_id = '.(int)$catid
					. ' AND c.access <= '.$gid
					. ' ORDER BY c.ordering'
					;
			$db->setQuery( $query );
			$cats = $db->loadObjectList();
		
			foreach($cats as $cat) {
				$node = new stdclass;
				$node->id   		= $parent->id;
				$node->uid  		= $parent->uid.'c'.$cat->id;
				$node->pid  		= $cat->parent_id;
				$node->name 		= $cat->title;
				$node->priority   	= $params['cat_priority'];
				$node->changefreq 	= $params['cat_changefreq'];
				$node->link 		= QuickfaqHelperRoute::getCategoryRoute( $cat->slug );
				$node->tree 		= array();

				//parents
				if ($xmap->printNode($node) !== FALSE) {
					xmap_com_quickfaq::getQuickfaqCatTree($xmap, $parent, $params, $cat->id);
				}
			}
		}
		
		if ($params['include_items']) {
		
			// First thing we need to do is to select only the requested items
			$where = ' WHERE rel.catid = '.(int)$catid;

			// Second is to only select items the user has access to
			$states = '1, -4';
			if ($user->authorize('com_quickfaq', 'state')) {
				$states .= ', 0 , -2, -3';
			}
			$where .= ' AND i.state IN ('.$states.')';
			$where .= ' AND c.published = 1';
			$where .= ' AND c.access <= '.$gid;
			
			$where .= $params['days'];
			
			$query = 'SELECT DISTINCT i.id, i.title, c.id AS cid,'
					. ' CASE WHEN CHAR_LENGTH(i.alias) THEN CONCAT_WS(\':\', i.id, i.alias) ELSE i.id END as slug,'
					. ' CASE WHEN CHAR_LENGTH( c.alias ) THEN CONCAT_WS( \':\', c.id, c.alias ) ELSE c.id END AS catslug'
					. ' FROM #__quickfaq_items AS i'
					. ' LEFT JOIN #__quickfaq_cats_item_relations AS rel ON rel.itemid = i.id'
					. ' LEFT JOIN #__quickfaq_categories AS c ON c.id = '. (int)$catid
					. $where
					. ' ORDER BY i.title'
					. $params['limit']
					;

			$db->setQuery ( $query );
			$items = $db->loadObjectList();
			
			foreach($items as $item) {
				$node = new stdclass;
				$node->id   		= $parent->id;
				$node->uid  		= $parent->uid .'d'.$item->id;
				$node->name 		= $item->title;
				$node->link 		= QuickfaqHelperRoute::getItemRoute( $item->slug, $item->catslug );
				$node->priority   	= $params['item_priority'];
				$node->changefreq 	= $params['item_changefreq'];
				$node->tree 		= array();
				$xmap->printNode($node);
			}
			
			$xmap->changeLevel(-1);
			
		//if items shouldn't shown, switch one level back here
		} else {
			$xmap->changeLevel(-1);
		}
	}
	
	/*
	 * Get the FAQItems tagged with the tag which is defined in the menuitem
	 */
	function getQuickfaqTagTree ( &$xmap, &$parent, &$params, &$tid ) {
		
		$db 		= &JFactory::getDBO();
		$user 		= &JFactory::getUser();
		$gid		= (int) $user->get('aid');
		
		$xmap->changeLevel(1);
				
		if ($params['include_items']) {
		
			// First thing we need to do is to select only the requested items
			$where = ' WHERE t.tid = '.(int)$tid;

			// Second is to only select items the user has access to
			$states = '1, -4';
			if ($user->authorize('com_quickfaq', 'state')) {
				$states .= ', 0 , -2, -3';
			}
			$where .= ' AND i.state IN ('.$states.')';
			$where .= ' AND c.published = 1';
			$where .= ' AND c.access <= '.$gid;
			
			$where .= $params['days'];
			
			$query = 'SELECT DISTINCT i.id, i.title, c.id AS cid,'
					. ' CASE WHEN CHAR_LENGTH(i.alias) THEN CONCAT_WS(\':\', i.id, i.alias) ELSE i.id END as slug,'
					. ' CASE WHEN CHAR_LENGTH( c.alias ) THEN CONCAT_WS( \':\', c.id, c.alias ) ELSE c.id END AS catslug'
					. ' FROM #__quickfaq_items AS i'
					. ' INNER JOIN #__quickfaq_tags_item_relations AS t ON t.itemid = i.id'
					. ' LEFT JOIN #__quickfaq_cats_item_relations AS rel ON rel.itemid = i.id'
					. ' LEFT JOIN #__quickfaq_categories AS c ON c.id = rel.catid'
					. $where
					. ' ORDER BY i.title'
					. $params['limit']
					;

			$db->setQuery ( $query );
			$items = $db->loadObjectList();
			
			foreach($items as $item) {
				$node = new stdclass;
				$node->id   		= $parent->id;
				$node->uid  		= $parent->uid .'d'.$item->id;
				$node->name 		= $item->title;
				$node->link 		= QuickfaqHelperRoute::getItemRoute( $item->slug, $item->catslug );
				$node->priority   	= $params['item_priority'];
				$node->changefreq 	= $params['item_changefreq'];
				$node->tree 		= array();
				$xmap->printNode($node);
			}
			
			$xmap->changeLevel(-1);
			
		//if items shouldn't shown, switch one level back here
		} else {
			$xmap->changeLevel(-1);
		}
	}
}
?>