<?php
/**
 * @version 0.9 $Id: quickfaq.php 3 2008-04-18 16:39:45Z schlu $
 * @package Joomla
 * @subpackage QuickFAQ search
 * @copyright (C) 2008 Christoph Lukes
 * @license GNU/GPL, see LICENCE.php
 * QuickFAQ search is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.

 * QuickFAQ search is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with QuickFAQ search; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

$mainframe->registerEvent( 'onSearch', 'plgSearchQuickfaq' );
$mainframe->registerEvent( 'onSearchAreas', 'plgSearchQuickfaqAreas' );

//Load the Plugin language file out of the administration
JPlugin::loadLanguage( 'plg_search_quickfaq', JPATH_ADMINISTRATOR);

/**
 * @return array An array of search areas
 */
function &plgSearchQuickfaqAreas() {
	static $areas = array(
	'quickfaq' => 'QUICKFAQ'
	);
	return $areas;
}

/**
 * Search method
 *
 * The sql must return the following fields that are
 * used in a common display routine: href, title, section, created, text,
 * browsernav
 * @param string Target search string
 * @param string mathcing option, exact|any|all
 * @param string ordering option, newest|oldest|popular|alpha|category
 * @param mixed An array if restricted to areas, null if search all
 */
function plgSearchQuickfaq( $text, $phrase='', $ordering='', $areas=null )
{
	$db		=& JFactory::getDBO();
	$user	=& JFactory::getUser();

	require_once(JPATH_SITE.DS.'components'.DS.'com_quickfaq'.DS.'helpers'.DS.'route.php');

	if (is_array( $areas )) {
		if (!array_intersect( $areas, array_keys( plgSearchQuickfaqAreas() ) )) {
			return array();
		}
	}

	// load plugin params info
	$plugin =& JPluginHelper::getPlugin('search', 'quickfaq');
	$pluginParams = new JParameter( $plugin->params );

	$limit = $pluginParams->def( 'search_limit', 50 );

	$text = trim( $text );
	if ( $text == '' ) {
		return array();
	}

	$text = $db->getEscaped($text);

	$searchQuickfaq = JText::_( 'QUICKFAQ' );

	switch ($phrase) {
		case 'exact':
			$text		= $db->Quote( '%'.$db->getEscaped( $text, true ).'%', false );
			$wheres2 	= array();
			$wheres2[] 	= 'LOWER(a.title) LIKE '.$text;
			$wheres2[] 	= 'LOWER(a.introtext) LIKE '.$text;
			$wheres2[] 	= 'LOWER(a.`fulltext`) LIKE '.$text;
			$wheres2[] 	= 'LOWER(a.meta_keywords) LIKE '.$text;
			$wheres2[] 	= 'LOWER(a.meta_description) LIKE '.$text;
			$where 		= '(' . implode( ') OR (', $wheres2 ) . ')';
			break;

		case 'all':
		case 'any':
		default:
			$words = explode( ' ', $text );
			$wheres = array();
			foreach ($words as $word) {
				$word		= $db->Quote( '%'.$db->getEscaped( $word, true ).'%', false );
				$wheres2 	= array();
				$wheres2[] 	= 'LOWER(a.title) LIKE '.$word;
				$wheres2[] 	= 'LOWER(a.introtext) LIKE '.$word;
				$wheres2[] 	= 'LOWER(a.`fulltext`) LIKE '.$word;
				$wheres2[] 	= 'LOWER(a.meta_keywords) LIKE '.$word;
				$wheres2[] 	= 'LOWER(a.meta_description) LIKE '.$word;
				$wheres[] 	= implode( ' OR ', $wheres2 );
			}
			$where = '(' . implode( ($phrase == 'all' ? ') AND (' : ') OR ('), $wheres ) . ')';
			break;
	}

	switch ( $ordering ) {
		case 'oldest':
			$order = 'a.created ASC';
			break;

		case 'popular':
			$order = 'a.hits DESC';
			break;

		case 'alpha':
			$order = 'a.title ASC';
			break;

		case 'category':
			$order = 'c.title ASC, a.title ASC';
			break;

		case 'newest':
		default:
			$order = 'a.created DESC';
			break;
	}
	
	$query = 'SELECT DISTINCT a.title AS title,'
	. ' a.created AS created,'
	. ' CONCAT(a.introtext, a.`fulltext`) AS text,'
	. ' "2" AS browsernav,'
	. ' CASE WHEN CHAR_LENGTH(a.alias) THEN CONCAT_WS(\':\', a.id, a.alias) ELSE a.id END as slug,'
	. ' CASE WHEN CHAR_LENGTH(c.alias) THEN CONCAT_WS(\':\', c.id, c.alias) ELSE c.id END as categoryslug,'
	. ' CONCAT_WS( " / ", '. $db->Quote($searchQuickfaq) .', c.title, a.title ) AS section'
	. ' FROM #__quickfaq_items AS a'
	. ' LEFT JOIN #__quickfaq_cats_item_relations AS rel ON rel.itemid = a.id'
	. ' LEFT JOIN #__quickfaq_categories AS c ON c.id = rel.catid'
	. ' WHERE ( '.$where.' )'
	. ' AND a.state IN (1, -4)'
	. ' AND c.published = 1'
	. ' AND c.access <= '.(int) $user->get('aid')
	. ' ORDER BY '. $order
	;

	$db->setQuery( $query, 0, $limit );
	$list = $db->loadObjectList();

	if(isset($list))
	{
		foreach($list as $key => $row) {
			$list[$key]->href = QuickfaqHelperRoute::getItemRoute($row->slug, $row->categoryslug);
		}
	}

	return $list;
}
?>